<?php
/**
 * Smart Wishlist Install
 *
 * @author MoreConvert
 * @package Smart Wishlist For More Convert
 * @version 1.2.2
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}


if ( ! class_exists( 'WLFMC_Install' ) ) {
	/**
	 * Install plugin table and create the wishlist page
	 *
	 */
	class WLFMC_Install {

		/**
		 * Single instance of the class
		 *
		 * @var WLFMC_Install
		 */
		protected static $instance;

		/**
		 * Items table name
		 *
		 * @var string
		 * @access private
		 */
		private $_table_items;

		/**
		 * Items table name
		 *
		 * @var string
		 * @access private
		 */
		private $_table_wishlists;

		/**
		 * Email table name
		 *
		 * @var string
		 * @access private
		 */
		private $_table_offers;

		/**
		 * Returns single instance of the class
		 *
		 * @return WLFMC_Install
		 */
		public static function get_instance() {
			if ( is_null( self::$instance ) ) {
				self::$instance = new self();
			}

			return self::$instance;
		}

		/**
		 * Constructor.
		 *
		 */
		public function __construct() {
			global $wpdb;

			// define local private attribute.
			$this->_table_items     = $wpdb->prefix . 'wlfmc_wishlist_items';
			$this->_table_wishlists = $wpdb->prefix . 'wlfmc_wishlists';
			$this->_table_offers    = $wpdb->prefix . 'wlfmc_wishlist_offers';

			// add custom field to global $wpdb.
			$wpdb->wlfmc_items     = $this->_table_items;
			$wpdb->wlfmc_wishlists = $this->_table_wishlists;
			$wpdb->wlfmc_offers    = $this->_table_offers;

		}

		/**
		 * Init db structure of the plugin
		 *
		 */
		public function init() {
			$this->_add_tables();
			$this->_add_pages();

			$this->register_current_version();
		}


		/**
		 * Register current version of plugin and database sctructure
		 */
		public function register_current_version() {
			delete_option( 'wlfmc_version' );
			update_option( 'wlfmc_version', WLFMC_VERSION );

			delete_option( 'wlfmc_db_version' );
			update_option( 'wlfmc_db_version', WLFMC_DB_VERSION );
		}

		/**
		 * Check if the table of the plugin already exists.
		 *
		 * @return bool
		 */
		public function is_installed() {
			global $wpdb;
			$number_of_tables = $wpdb->query( $wpdb->prepare( 'SHOW TABLES LIKE %s', $wpdb->prefix . "wlfmc_wishlist%" ) );

			return 3 == $number_of_tables;
		}

		/**
		 * Add tables for a fresh installation
		 *
		 * @return void
		 * @access private
		 *
		 * @version 1.0.1
		 */
		private function _add_tables() {
			if ( ! $this->is_installed() ) {
				$this->_add_wishlists_table();
				$this->_add_items_table();
				$this->_add_offer_table();
			}
		}

		/**
		 * Add the wishlists table to the database.
		 *
		 * @return void
		 * @access private
		 */
		private function _add_wishlists_table() {


			$sql = "CREATE TABLE {$this->_table_wishlists} (
							ID BIGINT( 20 ) NOT NULL AUTO_INCREMENT,
							user_id BIGINT( 20 ) NULL DEFAULT NULL,
							session_id VARCHAR( 255 ) DEFAULT NULL,
							wishlist_slug VARCHAR( 200 ) NOT NULL,
							wishlist_name TEXT,
							wishlist_token VARCHAR( 64 ) NOT NULL UNIQUE,
							wishlist_privacy TINYINT( 1 ) NOT NULL DEFAULT 0,
							is_default TINYINT( 1 ) NOT NULL DEFAULT 0,
							dateadded timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
							expiration timestamp NULL DEFAULT NULL,
							PRIMARY KEY  ( ID ),
							KEY wishlist_slug ( wishlist_slug )
						) DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;";

			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
			dbDelta( $sql );

			return;
		}

		/**
		 * Add the items' table to the database.
		 *
		 * @return void
		 * @access private
		 */
		private function _add_items_table() {


			$sql = "CREATE TABLE {$this->_table_items} (
							ID BIGINT( 20 ) NOT NULL AUTO_INCREMENT,
							prod_id BIGINT( 20 ) NOT NULL,
							quantity INT( 11 ) NOT NULL,
							user_id BIGINT( 20 ) NULL DEFAULT NULL,
							wishlist_id BIGINT( 20 ) NULL,
							position INT( 11 ) DEFAULT 0,
							note VARCHAR (250),
							importance TINYINT( 1 ) NOT NULL DEFAULT 0,
							original_price DECIMAL( 9,3 ) NULL DEFAULT NULL,
							original_currency CHAR( 3 ) NULL DEFAULT NULL,
							dateadded timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
							on_sale tinyint NOT NULL DEFAULT 0,
							PRIMARY KEY  ( ID ),
							KEY prod_id ( prod_id )
						) DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;";

			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
			dbDelta( $sql );

			return;
		}

		/**
		 * Add the email table to the database.
		 *
		 * @return void
		 * @access private
		 *
		 *
		 * @version 1.0.1
		 */
		private function _add_offer_table() {


			$sql = "CREATE TABLE {$this->_table_offers} (
							ID BIGINT( 20 ) NOT NULL AUTO_INCREMENT,
							user_id BIGINT( 20 ) NULL DEFAULT NULL,
							wishlist_id BIGINT( 20 ) NULL,
							has_coupon  TINYINT( 1 ) NOT NULL DEFAULT 0,
							coupon_id BIGINT( 20 ) NULL,
							product_id BIGINT( 20 ) NULL,
							email_options LONGTEXT,
							days SMALLINT( 3 ) NULL DEFAULT 0,
							dateadded timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
							datesend timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
							datesent timestamp NULL  ON UPDATE CURRENT_TIMESTAMP,
							is_sent TINYINT( 1 ) NOT NULL DEFAULT 0,
							PRIMARY KEY  ( ID )
						) DEFAULT CHARSET=utf8 AUTO_INCREMENT=1 ;";

			require_once ABSPATH . 'wp-admin/includes/upgrade.php';
			dbDelta( $sql );


			return;
		}

		/**
		 * Update db structure and options of the plugin
		 *
		 * @param string $current_version Version from which we're updating.
		 * @param string $current_db_version Version from which we're updating.
		 *
		 * @since 1.0.1
		 * @version 1.2.0
		 */
		public function update( $current_version, $current_db_version ) {

			if ( version_compare( $current_db_version, '1.0.1', '<' ) ) {
				$this->_update_1_0_1();
			}
			if ( version_compare( $current_version, '1.2.0', '<' ) ) {

				$this->_update_1_2_0();
			}
			if ( version_compare( $current_version, '1.2.2', '<' ) ) {

				$this->_update_1_2_2();
			}
			if ( version_compare( $current_version, '1.3.0', '<' ) ) {

				$this->_update_1_3_0();
			}
			if ( version_compare( $current_version, '1.3.1', '<' ) ) {

				$this->_update_1_3_1();
			}


			$this->register_current_version();
		}

		/**
		 * Update from 1.0.0 to 1.0.1
		 *
		 * @since 1.0.1
		 */
		private function _update_1_0_1() {
			global $wpdb;

			if ( $wpdb->get_var( "SHOW TABLES LIKE '{$wpdb->wlfmc_offers}';" ) ) {
				if ( ! $wpdb->get_var( "SHOW COLUMNS FROM `{$wpdb->wlfmc_offers}` LIKE 'days';" ) ) {
					$wpdb->query( "ALTER TABLE {$wpdb->wlfmc_offers} ADD `days` SMALLINT( 3 ) NULL DEFAULT 0;" );
				}
				if ( ! $wpdb->get_var( "SHOW COLUMNS FROM `{$wpdb->wlfmc_offers}` LIKE 'product_id';" ) ) {
					$wpdb->query( "ALTER TABLE {$wpdb->wlfmc_offers} ADD `product_id` BIGINT( 20 ) NULL;" );
				}
			}
		}

		/**
		 * Update to 1.2.0
		 *
		 * @since 1.2.0
		 */
		public function _update_1_2_0() {

			set_transient( '_wlfmc_wishlist_activation_redirect', true, 30 );

			update_option( 'smart-wishlist-for-more-convert_tracking_notice', 'hide' );

			update_option( 'wlfmc-skip-wizard-notice', true );

			update_option( 'wlfmc-finish-wizard-notice', true );

			// If our option doesn't exist already, we'll create it with today's timestamp.
			if ( ! get_option( 'wlfmc_wishlist_activation_date' ) ) {
				update_option( 'wlfmc_wishlist_activation_date', date( 'Y-m-d' ) );
			}
		}

		/**
		 * Update to 1.2.2
		 *
		 * @since 1.2.2
		 */
		public function _update_1_2_2() {
			$WLFMC_Offer_Email   = new WLFMC_Offer_Email();
			$default_Offer_Email = array(
				array(
					'enable_email'    => '1',
					'send_after_days' => '1',
					'mail_heading'    => __( 'Check it out, {user_name}', 'wc-wlfmc-wishlist' ),
					'mail_subject'    => __( 'Check it out, {user_name}', 'wc-wlfmc-wishlist' ),
					'html_content'    => $WLFMC_Offer_Email::get_default_content( 'html', 1 ),
					'text_content'    => $WLFMC_Offer_Email::get_default_content( 'plain', 1 ),
				),
				array(
					'enable_email'    => '1',
					'send_after_days' => '3',
					'mail_heading'    => __( 'Deals you’ve been waiting for!', 'wc-wlfmc-wishlist' ),
					'mail_subject'    => __( 'Deals you’ve been waiting for!', 'wc-wlfmc-wishlist' ),
					'html_content'    => $WLFMC_Offer_Email::get_default_content( 'html', 2 ),
					'text_content'    => $WLFMC_Offer_Email::get_default_content( 'plain', 2 ),
				),
				array(
					'enable_email'    => '1',
					'send_after_days' => '5',
					'mail_heading'    => __( 'Got {coupon_amount} off your favorites?', 'wc-wlfmc-wishlist' ),
					'mail_subject'    => __( 'Got {coupon_amount} off your favorites?', 'wc-wlfmc-wishlist' ),
					'html_content'    => $WLFMC_Offer_Email::get_default_content( 'html', 3 ),
					'text_content'    => $WLFMC_Offer_Email::get_default_content( 'plain', 3 ),
				),
				array(
					'enable_email'    => '1',
					'send_after_days' => '7',
					'mail_heading'    => __( 'The item on your wishlist is almost sold out!', 'wc-wlfmc-wishlist' ),
					'mail_subject'    => __( 'The item on your wishlist is almost sold out!', 'wc-wlfmc-wishlist' ),
					'html_content'    => $WLFMC_Offer_Email::get_default_content( 'html', 4 ),
					'text_content'    => $WLFMC_Offer_Email::get_default_content( 'plain', 4 ),
				),
				array(
					'enable_email'    => '1',
					'send_after_days' => '9',
					'mail_heading'    => __( 'Just one day for your wishlist', 'wc-wlfmc-wishlist' ),
					'mail_subject'    => __( 'Just one day for your wishlist', 'wc-wlfmc-wishlist' ),
					'html_content'    => $WLFMC_Offer_Email::get_default_content( 'html', 5 ),
					'text_content'    => $WLFMC_Offer_Email::get_default_content( 'plain', 5 ),
				),
			);
			$options             = new MCT_Options( 'wlfmc_options' );
			// back to default email offers .
			$options->update_option( 'offer_emails', $default_Offer_Email );
		}

		/**
		 * Update to 1.3.0
		 *
		 * @since 1.3.0
		 */
		public function _update_1_3_0() {

			$options = new MCT_Options( 'wlfmc_options' );
			if ( 'before_image' === $options->get_option( 'loop_position' ) ) {
				$options->update_option( 'loop_position', 'image_top_left' );
			}
			if ( 'thumbnails' === $options->get_option( 'wishlist_button_position' ) ) {
				$options->update_option( 'wishlist_button_position', 'image_top_left' );
			}
			if ( 'before_image' === $options->get_option( 'gutenberg_position' ) ) {
				$options->update_option( 'gutenberg_position', 'image_top_left' );
			}

			$options->update_option( 'enable_for_outofstock_product', '1' );
		}

		/**
		 * Update to 1.3.1
		 *
		 * @since 1.3.1
		 */
		public function _update_1_3_1() {

			$options = new MCT_Options( 'wlfmc_options' );
			$options->update_option( 'show_login_notice_for_guests', $options->get_option( 'show_login_notice_for_quests' ) );
		}


		/**
		 * Add a page "Wishlist".
		 *
		 * @return void
		 */
		private function _add_pages() {
			wc_create_page(
				sanitize_title_with_dashes( _x( 'wishlist', 'page_slug', 'wc-wlfmc-wishlist' ) ),
				'wlfmc_wishlist_page_id',
				__( 'Wishlist', 'wc-wlfmc-wishlist' ),
				'<!-- wp:shortcode -->[wlfmc_wishlist]<!-- /wp:shortcode -->'
			);
		}
	}
}

/**
 * Unique access to instance of WLFMC_Install class
 *
 * @return WLFMC_Install
 */
function WLFMC_Install() {
	return WLFMC_Install::get_instance();
}
